@@ -167,8 +167,8 @@ class _Project(_Base):
             for chunk in r.iter_content(chunk_size=8192): 
                 f.write(chunk)
 
-    def add_datasource_and_trainer(self, datasource_key, trainer_key):
-        return self._make_request('POST', '/bridges', params={'projectKey': self.key, 'datasourceKey': datasource_key, 'trainerKey': trainer_key})
+    def add_datasource_and_trainer(self, datasource_key, trainer_key, hyper_param_values={}):
+        return self._make_request('POST', '/bridges', params={'projectKey': self.key, 'datasourceKey': datasource_key, 'trainerKey': trainer_key, 'hyperParamValues': hyper_param_values})
 
 
 class _Datasource(_Base):
@@ -178,8 +178,8 @@ class _Datasource(_Base):
         self.key = key
         self.trainers = {}
 
-    def add_trainer(self, key, train_callback, test_callback, description=None, model_path=None):
-        params = {'key': key}
+    def add_trainer(self, key, train_callback, test_callback, default_hyper_params={} , description=None, model_path=None):
+        params = {'key': key , "defaultHyperParams": default_hyper_params}
         if description is not None:
             params['description'] = description
         self._make_request('POST', f'/datasources/{self.key}/trainers', params=params)
@@ -235,11 +235,13 @@ class DynamoFL(_Base):
             project_key = j['data']['projectKey']
             datasource_key = j['data']['datasourceKey']
             trainer_key = j['data']['trainerKey']
+            hyper_param_values = j['data']['hyperParamValues']
 
             self.project_participants.append({
                 'project_key': project_key,
                 'datasource_key': datasource_key,
-                'trainer_key': trainer_key
+                'trainer_key': trainer_key,
+                'hyper_param_values': hyper_param_values
             })
 
             info = self._make_request('GET', f'/projects/{project_key}')
@@ -261,6 +263,11 @@ class DynamoFL(_Base):
                 if project_key == p['project_key']:
                     self.on_round_threads[threads_key] = gevent.spawn(self.train_and_test_callback, p['datasource_key'], j['data']['project'])
 
+        elif j['event'] == 'hyperparams-updated':
+            for p in self.project_participants:
+                if project_key == p['project_key'] and p['datasource_key'] == j['data']['datasourceKey']:
+                    p['hyper_param_values'] = j['data']['hyperParamValues']
+
 
         elif j['event'] == 'round-error':
             for p in self.project_participants:
@@ -289,6 +296,7 @@ class DynamoFL(_Base):
         for p in self.project_participants:
             if project_info['key'] == p['project_key'] and datasource_key == p['datasource_key']:
                 trainer_key = p['trainer_key']
+                hyper_param_values = p['hyper_param_values']
                 break
 
         train = self.datasources[datasource_key].trainers[trainer_key]['train']
@@ -326,7 +334,7 @@ class DynamoFL(_Base):
             new_model_path = get_trained_path(model_path, ext, datasource_key, current_round)
 
             print(f'>>> ({project_key}-{datasource_key}) Training weights on local model...')
-            train(datasource_key, federated_model_path, new_model_path, project_info)
+            train(datasource_key, federated_model_path, new_model_path, project_info, hyper_param_values)
 
             print(f'>>> ({project_key}-{datasource_key}) Uploading round ({current_round}) trained model...')
             project.push_model(new_model_path, datasource_key)
